"""Provides Bootstrap3 and other static resources in a modular fashion.

https://github.com/Robpol86/Flask-Statics-Helper
https://pypi.python.org/pypi/Flask-Statics-Helper
"""

from flask import Blueprint

from flask_statics import resource_base
from flask_statics import resource_definitions
from flask_statics import resource_definitions_angular

__author__ = '@Robpol86'
__license__ = 'MIT'
__version__ = '0.1.1'


def priority(var):
    """Prioritizes resource position in the final HTML. To be fed into sorted(key=).

    Javascript consoles throw errors if Bootstrap's js file is mentioned before jQuery. Using this function such errors
    can be avoided. Used internally.

    Positional arguments:
    var -- value sent by list.sorted(), which is a value in Statics().all_variables.

    Returns:
    Either a number if sorting is enforced for the value in `var`, or returns `var` itself.
    """
    order = dict(JQUERY='0', BOOTSTRAP='1')
    return order.get(var, var)


class _StaticsState(object):
    """Remembers the configuration for the (statics, app) tuple. Modeled from SQLAlchemy."""

    def __init__(self, statics, app):
        self.statics = statics
        self.app = app


class Statics(object):
    """Static css/js resources for Flask applications.

    Relevant configuration settings from the Flask app config:
    STATICS_MINIFY -- set to True to have minified resources selected instead of uncompressed resources.

    Optional settings to enable specific static resources on all templates by default instead of on-demand:
    STATICS_ENABLE_RESOURCE_<resource name> -- refer to resource_definitions.py for list of options. Set to True to
        enable everywhere.
    """

    def __init__(self, app=None):
        self.all_variables = list()
        self.all_resources = dict()
        self.blueprint = None
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        """Initialize the extension."""
        # Set default Flask config option.
        app.config.setdefault('STATICS_MINIFY', False)

        # Populate resources
        subclasses = resource_base.ResourceBase.__subclasses__() + resource_definitions.ResourceAngular.__subclasses__()
        for resource in subclasses:
            obj = resource(app)
            self.all_resources[resource.RESOURCE_NAME] = dict(css=tuple(obj.resources_css), js=tuple(obj.resources_js))
        self.all_variables = sorted(self.all_resources.keys(), key=priority)

        # Add this instance to app.extensions.
        if not hasattr(app, 'extensions'):
            app.extensions = dict()
        app.extensions['statics'] = _StaticsState(self, app)

        # Initialize blueprint.
        name = 'flask_statics_helper'
        static_url_path = '{0}/{1}'.format(app.static_url_path, name)
        self.blueprint = Blueprint(name, __name__, template_folder='templates', static_folder='static',
                static_url_path=static_url_path)
        self.blueprint.add_app_template_global(self.all_variables, '_flask_statics_helper_all_variables')
        self.blueprint.add_app_template_global(self.all_resources, '_flask_statics_helper_all_resources')
        app.register_blueprint(self.blueprint)



"""Holds the base Resource class."""

import os


class ResourceBase(object):
    """Base class for loading resources (such as DataTables or Bootstrap).

    Class variables:
    DIR -- directory name under the static folder where files for this static resource reside.
    RESOURCE_NAME -- variable to set to True inside a template that wishes to use this resource (subclassing this
        class).

    Instance variables:
    minify -- Used to determine if minified resources should be selected in <link /> and <script /> tags or if developer
        non-minified resources should be selected instead. Default is non-minified.
    resources_css -- list of css files associated with this static resource.
    resources_js -- list of js files.
    """
    DIR = ''
    STATIC_DIR = os.path.join(os.path.dirname(__file__), 'static')
    RESOURCE_NAME = ''

    def __init__(self, app):
        self.minify = app.config['STATICS_MINIFY']
        self.resources_css = list()
        self.resources_js = list()

    def file_exists(self, subdir, prefix, suffix):
        """Returns true if the resource file exists, else False.

        Positional arguments:
        subdir -- sub directory name under the resource's main directory (e.g. css or js, or an empty string if the
            resource's directory structure is flat).
        prefix -- file name without the file extension.
        suffix -- file extension (if self.minify = True, includes .min before the extension).
        """
        real_path = os.path.join(self.STATIC_DIR, self.DIR, subdir, prefix + suffix)
        return os.path.exists(real_path)

    def add_css(self, subdir, file_name_prefix):
        """Add a css file for this resource.

        If self.minify is True, checks if the .min.css file exists. If not, falls back to non-minified file. If that
        file also doesn't exist, IOError is raised.

        Positional arguments:
        subdir -- sub directory name under the resource's main directory (e.g. css or js, or an empty string).
        file_name_prefix -- file name without the file extension.
        """
        suffix_maxify = '.css'
        suffix_minify = '.min.css'
        if self.minify and self.file_exists(subdir, file_name_prefix, suffix_minify):
            self.resources_css.append(os.path.join(self.DIR, subdir, file_name_prefix + suffix_minify))
        elif self.file_exists(subdir, file_name_prefix, suffix_maxify):
            self.resources_css.append(os.path.join(self.DIR, subdir, file_name_prefix + suffix_maxify))
        else:
            file_path = os.path.join(self.STATIC_DIR, self.DIR, subdir, file_name_prefix + suffix_maxify)
            raise IOError('Resource file not found: {}'.format(file_path))

    def add_js(self, subdir, file_name_prefix):
        """Same as self.add_css() but for js files."""
        suffix_maxify = '.js'
        suffix_minify = '.min.js'
        if self.minify and self.file_exists(subdir, file_name_prefix, suffix_minify):
            self.resources_js.append(os.path.join(self.DIR, subdir, file_name_prefix + suffix_minify))
        elif self.file_exists(subdir, file_name_prefix, suffix_maxify):
            self.resources_js.append(os.path.join(self.DIR, subdir, file_name_prefix + suffix_maxify))
        else:
            file_path = os.path.join(self.STATIC_DIR, self.DIR, subdir, file_name_prefix + suffix_maxify)
            raise IOError('Resource file not found: {}'.format(file_path))



"""Holds defined resources for AngularJS, subclassing resource_base.ResourceBase."""

from flask_statics.resource_definitions import ResourceAngular


class ResourceAngularAnimate(ResourceAngular):
    RESOURCE_NAME = 'ANGULARJS_ANIMATE'

    def __init__(self, app):
        super(ResourceAngularAnimate, self).__init__(app)
        self.add_js('', 'angular-animate')


class ResourceAngularCookies(ResourceAngular):
    RESOURCE_NAME = 'ANGULARJS_COOKIES'

    def __init__(self, app):
        super(ResourceAngularCookies, self).__init__(app)
        self.add_js('', 'angular-cookies')


class ResourceAngularCsp(ResourceAngular):
    RESOURCE_NAME = 'ANGULARJS_CSP'

    def __init__(self, app):
        super(ResourceAngularCsp, self).__init__(app)
        self.add_css('', 'angular-csp')


class ResourceAngularLoader(ResourceAngular):
    RESOURCE_NAME = 'ANGULARJS_LOADER'

    def __init__(self, app):
        super(ResourceAngularLoader, self).__init__(app)
        self.add_js('', 'angular-loader')


class ResourceAngularMessages(ResourceAngular):
    RESOURCE_NAME = 'ANGULARJS_MESSAGES'

    def __init__(self, app):
        super(ResourceAngularMessages, self).__init__(app)
        self.add_js('', 'angular-messages')


class ResourceAngularMocks(ResourceAngular):
    RESOURCE_NAME = 'ANGULARJS_MOCKS'

    def __init__(self, app):
        super(ResourceAngularMocks, self).__init__(app)
        self.add_js('', 'angular-mocks')


class ResourceAngularResource(ResourceAngular):
    RESOURCE_NAME = 'ANGULARJS_RESOURCE'

    def __init__(self, app):
        super(ResourceAngularResource, self).__init__(app)
        self.add_js('', 'angular-resource')


class ResourceAngularRoute(ResourceAngular):
    RESOURCE_NAME = 'ANGULARJS_ROUTE'

    def __init__(self, app):
        super(ResourceAngularRoute, self).__init__(app)
        self.add_js('', 'angular-route')


class ResourceAngularSanitize(ResourceAngular):
    RESOURCE_NAME = 'ANGULARJS_SANITIZE'

    def __init__(self, app):
        super(ResourceAngularSanitize, self).__init__(app)
        self.add_js('', 'angular-sanitize')


class ResourceAngularScenario(ResourceAngular):
    RESOURCE_NAME = 'ANGULARJS_SCENARIO'

    def __init__(self, app):
        super(ResourceAngularScenario, self).__init__(app)
        self.add_js('', 'angular-scenario')


class ResourceAngularTouch(ResourceAngular):
    RESOURCE_NAME = 'ANGULARJS_TOUCH'

    def __init__(self, app):
        super(ResourceAngularTouch, self).__init__(app)
        self.add_js('', 'angular-touch')



"""Holds defined resources, subclassing resource_base.ResourceBase."""

from flask_statics.resource_base import ResourceBase


class ResourceJQuery(ResourceBase):
    DIR = 'jquery-2.1.1'
    RESOURCE_NAME = 'JQUERY'

    def __init__(self, app):
        super(ResourceJQuery, self).__init__(app)
        self.add_js('', 'jquery-2.1.1')


class ResourceAngular(ResourceBase):
    DIR = 'angular-1.3.0-beta.14'
    RESOURCE_NAME = 'ANGULARJS'

    def __init__(self, app):
        super(ResourceAngular, self).__init__(app)
        self.add_js('', 'angular')


class ResourceBootstrap(ResourceBase):
    DIR = 'bootstrap-3.2.0'
    RESOURCE_NAME = 'BOOTSTRAP'

    def __init__(self, app):
        super(ResourceBootstrap, self).__init__(app)
        self.add_css('css', 'bootstrap')
        self.add_js('js', 'bootstrap')


class ResourceBootstrapEditable(ResourceBase):
    DIR = 'bootstrap3-editable-1.5.1'
    RESOURCE_NAME = 'BOOTSTRAP_EDITABLE'

    def __init__(self, app):
        super(ResourceBootstrapEditable, self).__init__(app)
        self.add_css('css', 'bootstrap-editable')
        self.add_js('js', 'bootstrap-editable')


class ResourceBootstrapGrowl(ResourceBase):
    DIR = 'bootstrap-growl-2.0.0'
    RESOURCE_NAME = 'BOOTSTRAP_GROWL'

    def __init__(self, app):
        super(ResourceBootstrapGrowl, self).__init__(app)
        self.add_js('', 'bootstrap-growl')


class ResourceBootstrapTypeahead(ResourceBase):
    DIR = 'Bootstrap-3-Typeahead-3.0.3'
    RESOURCE_NAME = 'BOOTSTRAP_TYPEAHEAD'

    def __init__(self, app):
        super(ResourceBootstrapTypeahead, self).__init__(app)
        self.add_js('', 'bootstrap3-typeahead')


class ResourceCSShake(ResourceBase):
    DIR = 'csshake-20140709'
    RESOURCE_NAME = 'CSSHAKE'

    def __init__(self, app):
        super(ResourceCSShake, self).__init__(app)
        self.add_css('', 'csshake')


class ResourceD3(ResourceBase):
    DIR = 'd3-3.4.9'
    RESOURCE_NAME = 'D3'

    def __init__(self, app):
        super(ResourceD3, self).__init__(app)
        self.add_js('', 'd3')


class ResourceDataTables(ResourceBase):
    DIR = 'DataTables-1.10.0'
    RESOURCE_NAME = 'DATATABLES'

    def __init__(self, app):
        super(ResourceDataTables, self).__init__(app)
        self.add_css('css', 'jquery.dataTables')
        self.add_js('js', 'jquery.dataTables')


class ResourceFontAwesome(ResourceBase):
    DIR = 'font-awesome-4.1.0'
    RESOURCE_NAME = 'FONT_AWESOME'

    def __init__(self, app):
        super(ResourceFontAwesome, self).__init__(app)
        self.add_css('css', 'font-awesome')


class ResourceWHHGFont(ResourceBase):
    DIR = 'whhg-font-20140709'
    RESOURCE_NAME = 'WHHG_FONT'

    def __init__(self, app):
        super(ResourceWHHGFont, self).__init__(app)
        self.add_css('css', 'whhg')
